/*  This file is part of Tapir.

    Tapir is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Tapir is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Tapir; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.elvum.tapir;
import javax.sound.sampled.*;
import java.nio.*;
import java.io.*;

class FileWriter {
	private AudioFormat m_audioFormat;
	private AudioInputStream m_audioInputStream;
	private SourceDataLine m_sourceDataLine;
	private float m_sampleRate = 44100.0F;
	private int m_sampleSizeInBits = 16;
	private int m_channels = 1;
	private boolean m_signed = true;
	private boolean m_bigEndian = true;
	private byte m_audioData[];
	private ByteBuffer m_byteBuffer;
	private ShortBuffer m_shortBuffer;
	private int m_byteLength;
	private File m_outputFile;

	protected FileWriter(String filename) {
		updateAudioFormat();
		m_outputFile = new File(filename);
	}
	
	public float sampleRate() {
		return m_sampleRate;
	}

	public void setSampleRate(float p_sampleRate) {
		m_sampleRate = p_sampleRate;
		updateAudioFormat();
	}

	public int sampleSizeInBits() {
		return m_sampleSizeInBits;
	}

	public void setSampleSizeInBits(int p_sampleSizeInBits) {
		m_sampleSizeInBits = p_sampleSizeInBits;
		updateAudioFormat();
	}

	public int channels() {
		return m_channels;
	}

	public void setChannels(int p_channels) {
		m_channels = p_channels;
		updateAudioFormat();
	}

	public boolean bigEndian() {
		return m_bigEndian;
	}

	public void setBigEndian(boolean p_bigEndian) {
		m_bigEndian = p_bigEndian;
		updateAudioFormat();
	}

	public boolean signed() {
		return m_signed;
	}

	public void setSigned(boolean p_signed) {
		m_signed = p_signed;
		updateAudioFormat();
	}

	void updateAudioFormat() {
		m_audioFormat = new AudioFormat(
			m_sampleRate,
			m_sampleSizeInBits,
			m_channels,
			m_signed,
			m_bigEndian
		);
	}

	public boolean write(short [] inputData) {
		m_audioData = new byte[inputData.length*2];
    m_byteBuffer = ByteBuffer.wrap(m_audioData);
    m_shortBuffer = m_byteBuffer.asShortBuffer();

		for (int i=0;i<inputData.length;i++) { //KLUDGE
			m_shortBuffer.put(inputData[i]);
		}

		try {
			InputStream byteArrayInputStream = new ByteArrayInputStream(m_audioData);
			m_audioInputStream = new AudioInputStream(byteArrayInputStream, m_audioFormat, m_audioData.length/m_audioFormat.getFrameSize());
			DataLine.Info dataLineInfo = new DataLine.Info(SourceDataLine.class, m_audioFormat);

			//Geta SourceDataLine object
			m_sourceDataLine = (SourceDataLine)AudioSystem.getLine(dataLineInfo);
			AudioSystem.write(m_audioInputStream, AudioFileFormat.Type.WAVE, m_outputFile);
		} catch (Exception e) {
				e.printStackTrace();
				return false;
		}		
		return true;
	}

	public boolean write(SymbolStream2bps symbolStream) {
		return write(symbolStream.getStream());
	}

}
